/**
 * Copyright (c) 2004, Paul Tuckey
 * All rights reserved.
 * ====================================================================
 * Licensed under the BSD License. Text as follows.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   - Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   - Neither the name tuckey.org nor the names of its contributors
 *     may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
package org.tuckey.utils;

import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;


/**
 * Log copies the style of commons logging.  The only reason this exists is that many problems
 * were had with Log4j and commons-logging interfering with peoples installed configuration.
 * It is very easy to change this to log4j or commons-logging in the future.
 *
 * @author Paul Tuckey
 * @version $Revision: 1.2 $ $Date: 2004/04/20 10:32:47 $
 */
public class Log {

    private static Log localLog = Log.getLog(Log.class);

    // static vars
    private static ServletContext context = null;
    private static final String DEFAULT_LOG_LEVEL = "INFO";
    private static String level = DEFAULT_LOG_LEVEL;
    private static boolean logToSystemOut = false;


    private Class clazz = null;

    public Log(Class clazz) {
        this.clazz = clazz;
    }

    public boolean isTraceEnabled() {
        if ("TRACE".equals(level)) return true;
        return false;
    }

    public boolean isDebugEnabled() {
        if ("TRACE".equals(level)) return true;
        if ("DEBUG".equals(level)) return true;
        return false;
    }

    public boolean isInfoEnabled() {
        if ("TRACE".equals(level)) return true;
        if ("DEBUG".equals(level)) return true;
        if ("INFO".equals(level)) return true;
        return false;
    }

    public boolean isWarnEnabled() {
        if ("TRACE".equals(level)) return true;
        if ("DEBUG".equals(level)) return true;
        if ("INFO".equals(level)) return true;
        if ("WARN".equals(level)) return true;
        return false;
    }

    public boolean isErrorEnabled() {
        if ("TRACE".equals(level)) return true;
        if ("DEBUG".equals(level)) return true;
        if ("INFO".equals(level)) return true;
        if ("WARN".equals(level)) return true;
        if ("ERROR".equals(level)) return true;
        return false;
    }

    public boolean isFatalEnabled() {
        if ("TRACE".equals(level)) return true;
        if ("DEBUG".equals(level)) return true;
        if ("INFO".equals(level)) return true;
        if ("WARN".equals(level)) return true;
        if ("ERROR".equals(level)) return true;
        if ("FATAL".equals(level)) return true;
        return true;
    }


    public void trace(Object o) {
        if (!isTraceEnabled()) {
            return;
        }
        write("TRACE", o);
    }

    public void trace(Object o, Throwable throwable) {
        if (!isTraceEnabled()) {
            return;
        }
        write("TRACE", o);
        write("TRACE", throwable);
    }

    public void debug(Object o) {
        if (!isDebugEnabled()) {
            return;
        }
        write("DEBUG", o);
    }

    public void debug(Object o, Throwable throwable) {
        if (!isDebugEnabled()) {
            return;
        }
        write("DEBUG", o);
        write("DEBUG", throwable);
    }

    public void info(Object o) {
        if (!isInfoEnabled()) {
            return;
        }
        write("INFO", o);
    }

    public void info(Object o, Throwable throwable) {
        if (!isInfoEnabled()) {
            return;
        }
        write("INFO", o);
        write("INFO", throwable);
    }

    public void warn(Object o) {
        if (!isWarnEnabled()) {
            return;
        }
        write("WARN", o);
    }

    public void warn(Object o, Throwable throwable) {
        if (!isWarnEnabled()) {
            return;
        }
        write("WARN", o);
        write("WARN", throwable);
    }

    public void error(Object o) {
        if (!isErrorEnabled()) {
            return;
        }
        write("ERROR", o);
    }

    public void error(Object o, Throwable throwable) {
        if (!isErrorEnabled()) {
            return;
        }
        write("ERROR", o);
        write("ERROR", throwable);
    }

    public void fatal(Object o) {
        if (!isFatalEnabled()) {
            return;
        }
        write("FATAL", o);
    }

    public void fatal(Object o, Throwable throwable) {
        if (!isFatalEnabled()) {
            return;
        }
        write("FATAL", o);
        write("FATAL", throwable);
    }

    /**
     * Will get an instance of log for a given class.
     *
     * @param aClass to log for
     * @return the log instance
     */
    public static Log getLog(Class aClass) {
        return new Log(aClass);
    }

    /**
     * Set the logging level (options are TRACE, DEBUG, INFO, WARN, ERROR, FATAL).
     *
     * @param level the level to log with
     */
    public static void setLevel(String level) {
        Log.level = level;
    }

    /**
     * Handles writing for throwable.
     *
     * @param level     log level to log for
     * @param throwable to log
     */
    private void write(String level, Throwable throwable) {
        write(level, "Exception: " + throwable.getMessage());
    }

    /**
     * Handles writing of log lines.
     *
     * @param level log level to log for
     * @param o     object to log (runs toString)
     */
    private void write(String level, Object o) {
        String msg = clazz.getName() + " " + level + ": " + o.toString();
        if (context == null) {
            System.out.println(msg);
        } else {
            context.log(msg);
            if (logToSystemOut) {
                System.out.println(msg);
            }
        }

    }

    /**
     * Resets log to default state.
     */
    public static void resetAll() {
        Log.context = null;
        Log.level = DEFAULT_LOG_LEVEL;
        Log.logToSystemOut = false;
    }

    /**
     * If set to true will cause the logger to log to system.out as well as context.log.
     *
     * @param logToSystemOut true or false
     */
    public static void setLogToSystemOut(boolean logToSystemOut) {
        Log.logToSystemOut = logToSystemOut;
    }

    /**
     * Will setup Log based on the filter config.  Uses init paramater "logLevel" to get the log level.
     * Defaults to "INFO".
     *
     * @param filterConfig the filter config to use
     */
    public static void setConfiguration(final FilterConfig filterConfig) {
        Log.context = filterConfig.getServletContext();

        String logLevelConf = filterConfig.getInitParameter("logLevel");
        localLog.trace("conf setting " + logLevelConf);
        if (logLevelConf != null && !"".equals(logLevelConf)) {
            Log.level = logLevelConf.toUpperCase();
        } else {
            Log.level = DEFAULT_LOG_LEVEL;
        }
    }

}
