/**
 * Copyright (c) 2004, Paul Tuckey
 * All rights reserved.
 * ====================================================================
 * Licensed under the BSD License. Text as follows.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   - Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   - Neither the name tuckey.org nor the names of its contributors
 *     may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
package org.tuckey.web.filters.urlrewrite;

import org.apache.oro.text.regex.MalformedPatternException;
import org.apache.oro.text.regex.Pattern;
import org.apache.oro.text.regex.Perl5Compiler;
import org.apache.oro.text.regex.Perl5Matcher;
import org.tuckey.utils.Log;
import org.tuckey.utils.NumberUtils;
import org.tuckey.utils.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.Calendar;
import java.util.Date;

/**
 * Conditions must be met when the filter is processing a url.
 *
 * @author Paul Tuckey
 * @version $Revision: 1.2 $ $Date: 2004/04/20 10:32:47 $
 */
public final class Condition {

    private static Log log = Log.getLog(Rule.class);

    /**
     * Used to compile the regular expression for value.
     */
    private final Perl5Compiler compiler = new Perl5Compiler();

    /**
     * Used for matching the regular expression against the header value.
     */
    private final Perl5Matcher matcher = new Perl5Matcher();

    /**
     * Used to identify the condition.
     */
    private int id = 0;

    /**
     * Used to cache the pattern for faster execution.
     */
    private Pattern pattern;

    /**
     * Type of condition ie, header, port etc.
     */
    private short type;

    /**
     * The name of the header etc.
     */
    private String name;

    /**
     * The operator that should be used to evaluate the condition.
     */
    private short operator;

    /**
     * Value of this condition if the type is header.
     */
    private String strValue;

    /**
     * Value of this condition if the type is port.
     */
    private long numericValue = 0;

    /**
     * What to do with the next rule, this will indicate "or" otherwise do "and".
     */
    private boolean processNextOr = false;

    /**
     * Error message from the regular expression compilation.
     */
    private String error = null;

    private boolean valid = false;
    private boolean initialised = false;

    // Type statics
    private static final short TYPE_HEADER = 1;
    private static final short TYPE_METHOD = 2;
    private static final short TYPE_PORT = 3;

    private static final short TYPE_TIME = 4;
    private static final short TYPE_TIME_YEAR = 5;
    private static final short TYPE_TIME_MONTH = 6;
    private static final short TYPE_TIME_DAY_OF_MONTH = 7;
    private static final short TYPE_TIME_DAY_OF_WEEK = 8;
    private static final short TYPE_TIME_AMPM = 9;
    private static final short TYPE_TIME_HOUR_OF_DAY = 10;
    private static final short TYPE_TIME_MINUTE = 11;
    private static final short TYPE_TIME_SECOND = 12;
    private static final short TYPE_TIME_MILLISECOND = 13;

    // Operators
    private static final short OPERATOR_EQUAL = 1;
    private static final short OPERATOR_NOT_EQUAL = 2;
    private static final short OPERATOR_GREATER_THAN = 3;
    private static final short OPERATOR_LESS_THAN = 4;
    private static final short OPERATOR_GREATER_THAN_OR_EQUAL = 5;
    private static final short OPERATOR_LESS_THAN_OR_EQUAL = 6;

    /**
     * Will check and see if the condition matches the request.
     *
     * @param hsRequest
     * @return true on match
     */
    public boolean matches(final HttpServletRequest hsRequest) {
        if (!initialised) {
            log.debug("condition not initialised skipping");
            // error initialising do not process
            return false;
        }
        if (!valid) {
            log.debug("condition not valid skipping");
            return false;
        }

        switch (type) {
            case TYPE_PORT:
                return evaluateNumericCondition(hsRequest.getServerPort());
            case TYPE_METHOD:
                return evaluateStringCondition(hsRequest.getMethod());

            case TYPE_TIME:
                return evaluateNumericCondition((new Date()).getTime());
            case TYPE_TIME_YEAR:
                return evaluateCalendarCondition(Calendar.YEAR);
            case TYPE_TIME_MONTH:
                return evaluateCalendarCondition(Calendar.MONTH);
            case TYPE_TIME_DAY_OF_MONTH:
                return evaluateCalendarCondition(Calendar.DAY_OF_MONTH);
            case TYPE_TIME_DAY_OF_WEEK:
                return evaluateCalendarCondition(Calendar.DAY_OF_WEEK);
            case TYPE_TIME_AMPM:
                return evaluateCalendarCondition(Calendar.AM_PM);
            case TYPE_TIME_HOUR_OF_DAY:
                return evaluateCalendarCondition(Calendar.HOUR_OF_DAY);
            case TYPE_TIME_MINUTE:
                return evaluateCalendarCondition(Calendar.MINUTE);
            case TYPE_TIME_SECOND:
                return evaluateCalendarCondition(Calendar.SECOND);
            case TYPE_TIME_MILLISECOND:
                return evaluateCalendarCondition(Calendar.MILLISECOND);

            default:
                return evaluateHeaderCondition(hsRequest);
        }
    }

    private boolean evaluateStringCondition(String value) {
        if (operator == OPERATOR_NOT_EQUAL) {
            return !matcher.matches(value, pattern);
        }
        return matcher.matches(value, pattern);
    }

    private boolean evaluateHeaderCondition(final HttpServletRequest hsRequest) {
        // type must be default... header
        final String headerValue = hsRequest.getHeader(name);
        if (log.isDebugEnabled()) {
            log.debug("evaluating request value " + headerValue + " against " + strValue);
        }
        if (headerValue == null) {
            // unable to get header value
            log.debug("unable to get header value");
            return false;
        } else {
            // must have a value try and match it
            return evaluateStringCondition(headerValue);
        }
    }

    /**
     * Will evaluate a calendar condition.
     *
     * @param calField the calendar field from Calendar
     * @return true or false
     */
    private boolean evaluateCalendarCondition(final int calField) {
        return evaluateNumericCondition((Calendar.getInstance()).get(calField));

    }

    /**
     * Will evaluate usign operator.
     *
     * @param compareWith what to compare with
     * @return true or false
     */
    private boolean evaluateNumericCondition(final long compareWith) {
        if (log.isDebugEnabled()) {
            log.debug("evaluating with operator, is " + compareWith + " " + getOperator() + " " + numericValue);
        }
        switch (operator) {
            case OPERATOR_NOT_EQUAL:
                return compareWith != numericValue;
            case OPERATOR_GREATER_THAN:
                return compareWith > numericValue;
            case OPERATOR_LESS_THAN:
                return compareWith < numericValue;
            case OPERATOR_GREATER_THAN_OR_EQUAL:
                return compareWith >= numericValue;
            case OPERATOR_LESS_THAN_OR_EQUAL:
                return compareWith <= numericValue;
            default:
                return compareWith == numericValue;
        }
    }


    /**
     * Returns false on failure. Use getError to get the description of the error.
     *
     * @return weather or not the condition was successful in initialisation.
     */
    public boolean initialise() {
        initialised = true;
        if (error != null) {
            return false;
        }
        switch (type) {

            case TYPE_METHOD:
                initStringValue();
                break;

            case TYPE_PORT:
                initNumericValue();
                break;

            case TYPE_TIME:
                initNumericValue();
                break;
            case TYPE_TIME_YEAR:
                initNumericValue();
                break;
            case TYPE_TIME_MONTH:
                initNumericValue();
                break;
            case TYPE_TIME_DAY_OF_MONTH:
                initNumericValue();
                break;
            case TYPE_TIME_DAY_OF_WEEK:
                initNumericValue();
                break;
            case TYPE_TIME_AMPM:
                initNumericValue();
                break;
            case TYPE_TIME_HOUR_OF_DAY:
                initNumericValue();
                break;
            case TYPE_TIME_MINUTE:
                initNumericValue();
                break;
            case TYPE_TIME_SECOND:
                initNumericValue();
                break;
            case TYPE_TIME_MILLISECOND:
                initNumericValue();
                break;

            default:
                // header
                initStringValue();
        }
        if (log.isDebugEnabled()) {
            log.debug("loaded condition " + getType() + " " + strValue);
        }
        valid = error == null;
        return valid;
    }

    private void initStringValue() {
        try {
            pattern = compiler.compile(strValue);
        } catch (MalformedPatternException e) {
            e.printStackTrace();
            setError("Problem compiling regular expression " + strValue + " (" + e.getMessage() + ")");
        }
    }

    /**
     * Will init a numeric value type ie port.
     */
    private void initNumericValue() {
        if (numericValue == 0) {
            numericValue = NumberUtils.stringToLong(strValue);
            if (numericValue == 0 && !"0".equals(strValue)) {
                setError("Value " + strValue + " is not a valid number (tried to cast to java type long)");
            }
        }
    }

    private void setError(String s) {
        log.error("Condition " + id + " had error: " + s);
        error = s;
    }

    /**
     * Will get the type code ie, method, port, header etc.
     *
     * @return String
     */
    public String getType() {
        switch (type) {
            case TYPE_METHOD:
                return "method";
            case TYPE_PORT:
                return "port";
            case TYPE_TIME:
                return "time";
            case TYPE_TIME_YEAR:
                return "year";
            case TYPE_TIME_MONTH:
                return "month";
            case TYPE_TIME_DAY_OF_MONTH:
                return "dayofmonth";
            case TYPE_TIME_DAY_OF_WEEK:
                return "dayofweek";
            case TYPE_TIME_AMPM:
                return "ampm";
            case TYPE_TIME_HOUR_OF_DAY:
                return "hourofday";
            case TYPE_TIME_MINUTE:
                return "minute";
            case TYPE_TIME_SECOND:
                return "second";
            case TYPE_TIME_MILLISECOND:
                return "millisecond";
            case TYPE_HEADER:
                return "header";
            default:
                return "";
        }
    }

    /**
     * Will set the type.
     *
     * @param strType the type
     */
    public void setType(final String strType) {
        if ("method".equals(strType)) {
            this.type = TYPE_METHOD;
        } else if ("port".equals(strType)) {
            this.type = TYPE_PORT;
        } else if ("time".equals(strType)) {
            this.type = TYPE_TIME;
        } else if ("year".equals(strType)) {
            this.type = TYPE_TIME_YEAR;
        } else if ("month".equals(strType)) {
            this.type = TYPE_TIME_MONTH;
        } else if ("dayofmonth".equals(strType)) {
            this.type = TYPE_TIME_DAY_OF_MONTH;
        } else if ("dayofweek".equals(strType)) {
            this.type = TYPE_TIME_DAY_OF_WEEK;
        } else if ("ampm".equals(strType)) {
            this.type = TYPE_TIME_AMPM;
        } else if ("hourofday".equals(strType)) {
            this.type = TYPE_TIME_HOUR_OF_DAY;
        } else if ("minute".equals(strType)) {
            this.type = TYPE_TIME_MINUTE;
        } else if ("second".equals(strType)) {
            this.type = TYPE_TIME_SECOND;
        } else if ("millisecond".equals(strType)) {
            this.type = TYPE_TIME_MILLISECOND;
        } else if ("header".equals(strType) || StringUtils.isBlank(strType)) {
            this.type = TYPE_HEADER;
        } else {
            setError("Type " + strType + " is not valid");
        }
    }

    /**
     * Will get the operator type.
     *
     * @return notequal, greater etc.
     */
    public String getOperator() {
        switch (operator) {
            case OPERATOR_NOT_EQUAL:
                return "notequal";
            case OPERATOR_GREATER_THAN:
                return "greater";
            case OPERATOR_LESS_THAN:
                return "less";
            case OPERATOR_GREATER_THAN_OR_EQUAL:
                return "greaterorequal";
            case OPERATOR_LESS_THAN_OR_EQUAL:
                return "lessorequal";
            case OPERATOR_EQUAL:
                return "equal";
            default:
                return "";
        }
    }

    /**
     * Will ste the operator.
     *
     * @param operator type
     */
    public void setOperator(final String operator) {
        if ("notequal".equals(operator)) {
            this.operator = OPERATOR_NOT_EQUAL;
        } else if ("greater".equals(operator)) {
            this.operator = OPERATOR_GREATER_THAN;
        } else if ("less".equals(operator)) {
            this.operator = OPERATOR_LESS_THAN;
        } else if ("greaterorequal".equals(operator)) {
            this.operator = OPERATOR_GREATER_THAN_OR_EQUAL;
        } else if ("lessorequal".equals(operator)) {
            this.operator = OPERATOR_LESS_THAN_OR_EQUAL;
        } else if ("equal".equals(operator) || StringUtils.isBlank(operator)) {
            this.operator = OPERATOR_EQUAL;
        } else {
            setError("Operator " + operator + " is not valid");
        }
    }

    /**
     * Will get the name.
     *
     * @return String
     */
    public String getName() {
        return name;
    }

    /**
     * Will set the name.
     *
     * @param name the name
     */
    public void setName(final String name) {
        this.name = name;
    }

    /**
     * Will return "add" or "or".
     *
     * @return "add" or "or"
     */
    public String getNext() {
        if (processNextOr) return "or";
        return "and";
    }

    /**
     * Will set next.
     *
     * @param next "or" or "and"
     */
    public void setNext(final String next) {
        if ("or".equals(next)) {
            this.processNextOr = true;
        } else if ("and".equals(next) || StringUtils.isBlank(next)) {
            this.processNextOr = false;
        } else {
            setError("Next " + next + " is not valid (can be 'and', 'or')");
        }
    }

    /**
     * Will get the value.
     *
     * @return String
     */
    public String getValue() {
        return strValue;
    }

    /**
     * Will set the value.
     *
     * @param value the value
     */
    public void setValue(final String value) {
        this.strValue = value;
    }

    /**
     * Will get the description of the error.
     *
     * @return String
     */
    public final String getError() {
        return error;
    }

    /**
     * True if process next is or.
     *
     * @return boolean
     */
    public boolean isProcessNextOr() {
        return processNextOr;
    }

    public void setId(int id) {
        this.id = id;
    }
}