/**
 * Copyright (c) 2004, Paul Tuckey
 * All rights reserved.
 * ====================================================================
 * Licensed under the BSD License. Text as follows.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   - Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   - Neither the name tuckey.org nor the names of its contributors
 *     may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
package org.tuckey.web.filters.urlrewrite;

import org.apache.commons.digester.Digester;
import org.tuckey.utils.Log;
import org.xml.sax.SAXParseException;

import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Configuration object for urlrewrite filter.
 *
 * @author Paul Tuckey
 * @version $Revision: 1.3 $ $Date: 2004/05/20 12:14:07 $
 */
public final class Conf {

    private static Log log = Log.getLog(Conf.class);

    private Digester confDigester;
    private final List errors = new ArrayList();
    private final List rules = new ArrayList(50);
    private boolean ok = false;
    private Date loadedDate = null;
    private int ruleIdCounter = 0;

    private static final String DTD_PUBLIC_PATH = "-//tuckey.org//DTD UrlRewrite 1.0//EN";
    private static final String DTD_PATH = "/org/tuckey/dtds/urlrewrite1.dtd";
    private String fileName;

    public Conf() {
        // empty const for testing
        loadedDate = new Date();
    }

    /**
     *
     */
    public Conf(final InputStream inputStream, String fileName) {
        setupConfDigester();
        load(inputStream);
        initialise();
        loadedDate = new Date();
        this.fileName = fileName;
    }

    /**
     *
     */
    private void setupConfDigester() {

        confDigester = new Digester();
        confDigester.setValidating(true);

        URL dtdUrl;
        dtdUrl = Conf.class.getResource(DTD_PATH);
        if (dtdUrl == null) {
            addError(DTD_PATH + " could not be found in jar.");
            return;
        }
        confDigester.register(DTD_PUBLIC_PATH, dtdUrl.toString());

        confDigester.addObjectCreate("urlrewrite/rule", Rule.class);
        confDigester.addSetNext("urlrewrite/rule", "addRule");

        confDigester.addSetProperties("urlrewrite/rule", "enabled", "enabled");

        confDigester.addCallMethod("urlrewrite/rule/name", "setName", 0);
        confDigester.addCallMethod("urlrewrite/rule/note", "setNote", 0);
        confDigester.addCallMethod("urlrewrite/rule/from", "setFrom", 0);

        confDigester.addCallMethod("urlrewrite/rule/to", "setTo", 0);
        confDigester.addSetProperties("urlrewrite/rule/to", "last", "toLast");
        confDigester.addSetProperties("urlrewrite/rule/to", "type", "toType");

        confDigester.addObjectCreate("urlrewrite/rule/condition", Condition.class);
        confDigester.addSetNext("urlrewrite/rule/condition", "addCondition");

        confDigester.addCallMethod("urlrewrite/rule/condition", "setValue", 0);
        confDigester.addSetProperties("urlrewrite/rule/condition", "name", "name");
        confDigester.addSetProperties("urlrewrite/rule/condition", "type", "type");
        confDigester.addSetProperties("urlrewrite/rule/condition", "next", "next");

        confDigester.addObjectCreate("urlrewrite/rule/set", SetAttribute.class);
        confDigester.addSetNext("urlrewrite/rule/set", "addSetAttribute");

        confDigester.addCallMethod("urlrewrite/rule/set", "setValue", 0);
        confDigester.addSetProperties("urlrewrite/rule/set", "type", "type");
        confDigester.addSetProperties("urlrewrite/rule/set", "name", "name");

        confDigester.push(this);
    }

    /**
     * @param inputStream stream of the conf file to load
     */
    private synchronized void load(final InputStream inputStream) {
        if (inputStream == null) {
            log.error("inputstream is null");
            return;
        }
        try {
            log.debug("about to push conf into digester");
            confDigester.parse(inputStream);

        } catch (SAXParseException e) {
            addError("Parse error on line " + e.getLineNumber() + " " + e.getMessage(), e);
            return;
        } catch (Exception e) {
            addError("Exception loading conf " + " " + e.getMessage(), e);
            return;
        }
    }

    /**
     * Initialise the conf file.  This will run initialise on each rule and condition in the conf file.
     */
    public void initialise() {
        if (log.isDebugEnabled()) {
            log.debug("now initialising conf");
        }
        boolean rulesOk = true;
        for (int i = 0; i < rules.size(); i++) {
            final Rule rule = (Rule) rules.get(i);
            if (!rule.initialise()) {
                // if we failed to initialise anything set the status to bad
                addError("rule " + rule.getDisplayName() + " failed to initialise");
                rulesOk = false;
            }
        }
        if (rulesOk) {
            ok = true;
        }
    }

    /**
     * Will add the rule to the rules list.
     *
     * @param rule The Rule to add
     */
    public void addRule(final Rule rule) {
        ruleIdCounter++;
        rule.setId(ruleIdCounter);
        rules.add(rule);
    }

    /**
     * Will get the List of errors.
     *
     * @return the List of errors
     */
    public List getErrors() {
        return errors;
    }

    /**
     * Will get the List of rules.
     *
     * @return the List of rules
     */
    public List getRules() {
        return rules;
    }


    /**
     * true if the conf has been loaded ok.
     *
     * @return boolean
     */
    public boolean isOk() {
        return ok;
    }

    private void addError(final String errorMsg) {
        errors.add(errorMsg);
        log.error(errorMsg);
    }

    private void addError(final String errorMsg, final Exception e) {
        errors.add(errorMsg);
        log.error(errorMsg, e);
    }

    public Date getLoadedDate() {
        return loadedDate;
    }

    public String getFileName() {
        return fileName;
    }

}
