/**
 * Copyright (c) 2004, Paul Tuckey
 * All rights reserved.
 * ====================================================================
 * Licensed under the BSD License. Text as follows.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   - Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   - Neither the name tuckey.org nor the names of its contributors
 *     may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * 20040304 - Thanks to Scott Askew for help with concurrency issue with Perl5Compiler and Perl5Matcher
 */
package org.tuckey.web.filters.urlrewrite;

import org.apache.oro.text.regex.*;
import org.tuckey.utils.Log;
import org.tuckey.utils.StringUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

/**
 * Defines a rule that can be run against an incoming request.
 *
 * @author Paul Tuckey
 * @version $Revision: 1.3 $ $Date: 2004/05/20 12:14:08 $
 */
public final class Rule {

    private static Log log = Log.getLog(Rule.class);

    private int id;

    private boolean enabled = true;
    private boolean initialised = false;
    private boolean valid = false;

    private String name;
    private String note;
    private String from;
    private String to;
    private short toType = 1;  // default to passthrough
    private boolean last = true;
    private int conditionIdCounter = 0;

    private static final short TO_TYPE_REDIRECT = 0;
    private static final short TO_TYPE_FORWARD = 1;
    private static final short TO_TYPE_INCLUDE = 2;
    private static final short TO_TYPE_PERMANENT_REDIRECT = 3;
    private static final short TO_TYPE_TEMPORARY_REDIRECT = 4;

    private Pattern pattern;
    private Perl5Substitution substitution;
    private final List errors = new ArrayList();
    private final List conditions = new ArrayList();
    private final List setAttributes = new ArrayList();

    /**
     * Constructor.
     */
    public Rule() {
        // empty
    }

    /**
     * Will run the rule against the uri and perform action required will return false is not matched
     * otherwise true.
     *
     * @param url
     * @param hsRequest
     * @return String of the rewritten url or the same as the url passed in if no match was made
     */
    public RewrittenUrl execute(final String url, final HttpServletRequest hsRequest,
                                final HttpServletResponse hsResponse) {
        // make sure all the conditions match
        if (log.isDebugEnabled()) {
            log.debug("rule " + id + " run called with " + url);
        }
        if (!valid) {
            log.debug("not valid, skipping");
            return null;
        }
        if (!initialised) {
            log.debug("not initialised, skipping");
            return null;
        }
        if (!enabled) {
            log.debug("not enabled, skipping");
            return null;
        }

        // apply the from and to
        String toUrl = null;
        final Perl5Matcher matcher = new Perl5Matcher();
        if ( substitution == null ) {
            if ( ! matcher.matches(url, pattern) ) {
                log.debug("no substitution and no match");
                return null;
            }
        }   else {
            toUrl = Util.substitute(matcher, pattern, substitution, url);
            if (toUrl.equals(url)) {
                log.debug("no match");
                return null;
            }
        }

        boolean lastMatches = true;
        boolean lastProcessNextOr = false;
        for (int i = 0; i < conditions.size(); i++) {
            final Condition condition = (Condition) conditions.get(i);
            final boolean matches = condition.matches(hsRequest);
            if (lastProcessNextOr) {
                if (!matches) return null;
            } else {
                // must be "and" fail if the last one was false
                if (!lastMatches) return null;
            }
            lastMatches = matches;
            lastProcessNextOr = condition.isProcessNextOr();
        }

        // make sure the setAttributes are handled
        if ( setAttributes.size() > 0 ) {
            for (int i = 0; i < setAttributes.size(); i++) {
                SetAttribute setAttribute = (SetAttribute) setAttributes.get(i);
                setAttribute.execute(matcher, pattern, url, hsRequest, hsResponse);
            }
        }

        // we have a match now handle the to
        return rewriteRequest(toUrl);
    }


    /**
     * Will perform the action defined by the rule ie, redirect or passthrough.
     *
     * @param toUrl
     */
    private RewrittenUrl rewriteRequest(final String toUrl) {

        RewrittenUrl rewrittenRequest = new RewrittenUrl(toUrl);

        // when there is at least one 'set', 'to' may be blank
        if ( setAttributes.size() > 0 && to == null ) {
            return rewrittenRequest;
        }
        if (toType == TO_TYPE_REDIRECT) {
            if (log.isDebugEnabled()) {
                log.debug("needs to be redirected to " + toUrl);
            }
            rewrittenRequest.setRedirect(true);

        } else if (toType == TO_TYPE_PERMANENT_REDIRECT) {
            if (log.isDebugEnabled()) {
                log.debug("needs to be permanentely redirected to " + toUrl);
            }
            rewrittenRequest.setPermanentRedirect(true);

        } else if (toType == TO_TYPE_TEMPORARY_REDIRECT) {
            if (log.isDebugEnabled()) {
                log.debug("needs to be temporarily redirected to " + toUrl);
            }
            rewrittenRequest.setTemporaryRedirect(true);

        } else if (toType == TO_TYPE_INCLUDE) {
            if (log.isDebugEnabled()) {
                log.debug(toUrl + " needs to be included");
            }
            rewrittenRequest.setInclude(true);

        } else if (toType == TO_TYPE_FORWARD) {

            // pass on to the the request on to the to url
            if (log.isDebugEnabled()) {
                log.debug("needs to be forwarded to " + toUrl);
            }
            rewrittenRequest.setForward(true);
        }
        return rewrittenRequest;
    }


    /**
     * Will initialise the rule.
     *
     * @return true on success
     */
    public boolean initialise() {
        // check all the conditions
        initialised = true;
        boolean ok = true;
        for (int i = 0; i < conditions.size(); i++) {
            final Condition condition = (Condition) conditions.get(i);
            if (!condition.initialise()) {
                log.error("Condition of type " + condition.getType() + " failed to initialise: " + condition.getError());
                ok = false;
            }
        }
        for (int i = 0; i < setAttributes.size(); i++) {
            final SetAttribute setAttribute = (SetAttribute) setAttributes.get(i);
            if (!setAttribute.initialise()) {
                log.error("set of type " + setAttribute.getType() + " failed to initialise: " + setAttribute.getError());
                ok = false;
            }
        }
        // compile the from regexp
        if (StringUtils.isBlank(from)) {
            addError("from is not valid because it is blank");
        } else {
            try {
                final Perl5Compiler compiler = new Perl5Compiler();
                pattern = compiler.compile(from, Perl5Compiler.READ_ONLY_MASK);
            } catch (MalformedPatternException e) {
                addError("from (" + from + ") is an invalid regular expression - " + e.getMessage());
            }
        }
        // set the substitution
        if (StringUtils.isBlank(to) && setAttributes.size() == 0 ) {
            addError("to is not valid because it is blank");
        } else if ( !StringUtils.isBlank(to)) {
            substitution = new Perl5Substitution(to);
        }
        if (!ok) {
            log.debug("failed to load rule");
        } else {
            log.debug("loaded rule " + getDisplayName() + " (" + from + ", " + to + " " + toType + ")");
        }
        if (errors.size() > 0) {
            ok = false;
        }
        valid = ok;
        return ok;
    }

    /**
     * Will get the contents of the from element.
     *
     * @return the contents of the from element
     */
    public String getFrom() {
        return from;
    }

    /**
     * Will set from, usually called by Digester.
     *
     * @param from the url to match from
     */
    public void setFrom(final String from) {
        this.from = from;
    }

    /**
     * Will set the to, usually called by Digester.
     *
     * @param to url for redirecting/passing through to
     */
    public void setTo(final String to) {
        if ( ! StringUtils.isBlank(to)) {
            this.to = to;
        }
    }

    /**
     * Redirect or passthrough, passthrough is the default.
     *
     * @param toTypeStr to type string
     */
    public void setToType(final String toTypeStr) {
        if ("redirect".equals(toTypeStr)) {
            toType = TO_TYPE_REDIRECT;
        } else if ("permanent-redirect".equals(toTypeStr)) {
            toType = TO_TYPE_PERMANENT_REDIRECT;
        } else if ("temporary-redirect".equals(toTypeStr)) {
            toType = TO_TYPE_TEMPORARY_REDIRECT;
        } else if ("include".equals(toTypeStr)) {
            toType = TO_TYPE_INCLUDE;
        } else if ("forward".equals(toTypeStr) || "passthrough".equals(toTypeStr) ||
                StringUtils.isBlank(toTypeStr)) {
            toType = TO_TYPE_FORWARD;
        } else {
            addError("type (" + toTypeStr + ") is not valid");
        }
    }

    /**
     * Will get to type ie redirect or passthrough.
     *
     * @return String
     */
    public String getToType() {
        if (toType == TO_TYPE_REDIRECT) return "redirect";
        if (toType == TO_TYPE_PERMANENT_REDIRECT) return "permanent-redirect";
        if (toType == TO_TYPE_TEMPORARY_REDIRECT) return "temporary-redirect";
        if (toType == TO_TYPE_REDIRECT) return "redirect";
        if (toType == TO_TYPE_INCLUDE) return "include";
        return "forward";
    }

    /**
     * Set to type.
     *
     * @param lastStr true or false
     */
    public void setToLast(final String lastStr) {
        if ("false".equals(lastStr))
            last = false;
        else
            last = true;
    }

    /**
     * Is this rule last?.
     *
     * @return boolean
     */
    public boolean isLast() {
        return last;
    }

    /**
     * Get to.
     *
     * @return String
     */
    public String getTo() {
        return to;
    }

    /**
     * Will get the rule's id.
     *
     * @return int
     */
    public int getId() {
        return id;
    }


    private void addError(String s) {
        log.error("Rule " + getDisplayName() + " had error: " + s);
        errors.add(s);
    }

    /**
     * Will get the list of errors.
     *
     * @return the list of errors
     */
    public List getErrors() {
        return errors;
    }

    /**
     * Will add the condition to the List.
     *
     * @param condition The Condition object to add
     */
    public void addCondition(final Condition condition) {
        conditionIdCounter++;
        conditions.add(condition);
        condition.setId(conditionIdCounter);
    }

    /**
     * Will add the SetAttribute to the List.
     *
     * @param setAttribute The SetAttribute object to add
     */
    public void addSetAttribute(final SetAttribute setAttribute) {
        setAttributes.add(setAttribute);
    }

    public List getSetAttributes() {
        return setAttributes;
    }

    /**
     * Will get the List of conditions.
     *
     * @return the List of Condition objects
     */
    public List getConditions() {
        return conditions;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getNote() {
        return note;
    }

    public void setNote(String note) {
        this.note = note;
    }

    public boolean isEnabled() {
        return enabled;
    }

    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }

    public void setId(int id) {
        this.id = id;
    }

    public String getDisplayName() {
        if (name != null) {
            return name + " (rule " + id + ")";
        }
        return "Rule " + id;
    }
}


