/**
 * Copyright (c) 2004, Paul Tuckey
 * All rights reserved.
 * ====================================================================
 * Licensed under the BSD License. Text as follows.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   - Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   - Neither the name tuckey.org nor the names of its contributors
 *     may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
package org.tuckey.web.filters.urlrewrite;

import org.tuckey.utils.FormattedCharArrayWriter;
import org.tuckey.utils.StringUtils;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;

/**
 * Outputs information about urlrewritefilter in plain ascii style, ie no HTML.
 *
 * @author Paul Tuckey
 * @version $Revision: 1.3 $ $Date: 2004/05/25 07:36:39 $
 */
public class Status {


    public static void displayStatus(final FormattedCharArrayWriter writer, final HttpServletRequest hsRequest,
                                     final UrlRewriter urlRewriter)
            throws IOException {

        Conf conf = urlRewriter.getConf();

        writer.writeln(UrlRewriteFilter.class.getName() + " version " + UrlRewriteFilter.VERSION);

        if (!conf.isOk()) {
            writer.heading("ERROR UrlRewriteFilter NOT ACTIVE");
        }

        final List errors = conf.getErrors();
        if (errors.size() > 0) {
            writer.heading("Errors During Configuration file Load");
            for (int i = 0; i < errors.size(); i++) {
                final String error = (String) errors.get(i);
                writer.writeln(error);
            }
        }

        showConf(conf, writer);

        showRequestInfo(writer, hsRequest);

    }

    public static void showConf(Conf conf, final FormattedCharArrayWriter writer) throws IOException {
        final List rules = conf.getRules();
        if (rules.size() > 0) {
            Date loaded = conf.getLoadedDate();
            SimpleDateFormat s = new SimpleDateFormat();
            writer.heading("Configuration (loaded " + s.format(loaded) + ")");
            for (int i = 0; i < rules.size(); i++) {
                final Rule rule = (Rule) rules.get(i);
                writer.nl();
                writer.writeln(rule.getDisplayName() +
                        (rule.isEnabled() ? "" : " **DISABLED**"));

                if (!StringUtils.isBlank(rule.getNote())) {
                    writer.writeln("  note: " + rule.getNote());
                }

                final List conditions = rule.getConditions();
                for (int j = 0; j < conditions.size(); j++) {
                    final Condition condition = (Condition) conditions.get(j);
                    writer.writeln("  condition: type " + condition.getType() +
                            " name " + condition.getName() +
                            " value " + condition.getValue() +
                            " next " + condition.getNext() +
                            " operator " + condition.getOperator());
                    final String condError = condition.getError();
                    if (condError != null) {
                        writer.writeln("    ERROR: " + condError);
                    }
                }

                writer.writeln("  from: " + rule.getFrom());
                if (!StringUtils.isBlank(rule.getTo())) {
                    writer.writeln("  to: " + rule.getToType() + " " + rule.getTo() +
                            " (" + (rule.isLast() ? "is last" : "not last") + ")");
                }
                final List setAttributes = rule.getSetAttributes();
                for (int j = 0; j < setAttributes.size(); j++) {
                    final SetAttribute setAttribute = (SetAttribute) setAttributes.get(j);
                    writer.writeln("  set: type " + setAttribute.getType() +
                            " name " + setAttribute.getName() +
                            " value " + setAttribute.getValue());
                    final String setError = setAttribute.getError();
                    if (setError != null) {
                        writer.writeln("    ERROR: " + setError);
                    }
                }
                final List ruleErrors = rule.getErrors();
                for (int j = 0; j < ruleErrors.size(); j++) {
                    writer.writeln("  ERROR: " + ruleErrors.get(j));
                }

            }
        }
    }

    private static void showRequestInfo(final FormattedCharArrayWriter writer, final HttpServletRequest hsRequest) throws IOException {
        // other info
        writer.heading("Generic Info");
        writer.writeln("method: " + hsRequest.getMethod());
        writer.writeln("protocol: " + hsRequest.getProtocol());
        writer.writeln("port: " + hsRequest.getServerPort());

        // show headers from request
        writer.heading("Your Headers");
        final Enumeration headers = hsRequest.getHeaderNames();
        while (headers.hasMoreElements()) {
            final String headerName = (String) headers.nextElement();
            // ignore cookies as they are handled later
            if ("cookie".equals(headerName)) continue;
            writer.writeln(headerName + ": " + hsRequest.getHeader(headerName));
        }

        writer.heading("Cookies");
        final Cookie[] cookies = hsRequest.getCookies();
        if (cookies != null) {
            if (cookies.length > 0) {
                for (int i = 0; i < cookies.length; i++) {
                    writer.writeln("cookie " + i);
                    final Cookie cookie = cookies[i];
                    writer.writeln("    name     : " + cookie.getName());
                    writer.writeln("    value    : " + cookie.getValue());
                    writer.writeln("    path     : " + cookie.getPath());
                    writer.writeln("    domain   : " + cookie.getDomain());
                    writer.writeln("    max age  : " + cookie.getMaxAge());
                    writer.writeln("    is secure: " + cookie.getSecure());
                    writer.writeln("    version  : " + cookie.getVersion());
                    writer.writeln("    comment  : " + cookie.getComment());
                }
            } else {
                writer.writeln("no cookies");
            }
        } else {
            writer.writeln("no cookies");
        }
    }


}



