/**
 * Copyright (c) 2004, Paul Tuckey
 * All rights reserved.
 * ====================================================================
 * Licensed under the BSD License. Text as follows.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   - Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   - Neither the name tuckey.org nor the names of its contributors
 *     may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
package org.tuckey.web.filters.urlrewrite;

import org.tuckey.utils.FormattedCharArrayWriter;
import org.tuckey.utils.StringUtils;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;

/**
 * Outputs information about urlrewritefilter in plain ascii style, ie no HTML.
 *
 * @author Paul Tuckey
 * @version $Revision: 1.2 $ $Date: 2004/04/20 10:32:48 $
 */
public class StatusHtml {

    StringBuffer buffer = new StringBuffer();

    public void displayStatus(final FormattedCharArrayWriter writer, final HttpServletRequest hsRequest,
                              final UrlRewriter urlRewriter)
            throws IOException {

        Conf conf = urlRewriter.getConf();


        println("<h1>" + UrlRewriteFilter.class.getName() + " version " + UrlRewriteFilter.VERSION + "</h1>");

        if (!conf.isOk()) {
            println("<h2 class=\"err\">ERROR UrlRewriteFilter NOT ACTIVE</h2>");
        }

        final List errors = conf.getErrors();
        if (errors.size() > 0) {
            println("<p class=\"err\">Errors During Configuration file Load</p>");
            println("<ul>");
            for (int i = 0; i < errors.size(); i++) {
                final String error = (String) errors.get(i);
                println("<li>" + error + "</li>");
            }
        }

        showConf(conf);


        showRequestInfo(writer, hsRequest);

    }

    private void println(String s) {
        buffer.append(s + "\n");
    }


    public void showConf(Conf conf) throws IOException {
        final List rules = conf.getRules();
        if (rules.size() > 0) {
            Date loaded = conf.getLoadedDate();
            SimpleDateFormat s = new SimpleDateFormat();
            println("<h2>Configuration</h2>");
            println("<p>Loaded from <code>" + conf.getFileName() + "</code> " + s.format(loaded) + "</p>");
            for (int i = 0; i < rules.size(); i++) {
                final Rule rule = (Rule) rules.get(i);
                println("<h3>" + rule.getDisplayName() +
                        (rule.isEnabled() ? "" : " **DISABLED**") + "</h3>");
                if (!StringUtils.isBlank(rule.getNote())) {
                    println("<dl><dd><p>" + StringUtils.nl2br(rule.getNote()) + "</p></dd></dl>");
                }
                println("<p>URL's matching <code>" + rule.getFrom() + "</code> will " +
                        ("forward".equals(rule.getToType()) ? "be <code>forwarded</code> to" : "") +
                        ("include".equals(rule.getToType()) ? "<code>include</code>" : "") +
                        ("redirect".equals(rule.getToType()) ? "be <code>redirected</code> to" : "") +
                        " <code>" + rule.getTo() + "</code>.</p>");
                if (rule.getConditions().size() > 0) {
                    List conditions = rule.getConditions();
                    println("<p>Given that the following condtion" +
                            (conditions.size() == 1 ? " is" : "s are") + " met.</p>" +
                            "<ol>");
                    for (int j = 0; j < conditions.size(); j++) {
                        Condition condition = (Condition) conditions.get(j);
                        println("<li>");
                        if ("header".equals(condition.getType())) {
                            println("The <code>" + condition.getName() + "</code> HTTP header " +
                                    ("notequal".equals(condition.getOperator()) ? "does NOT match" : "matches") + " " +
                                    "<code>" + condition.getValue() + "</code>");
                        } else {
                            println("<code>" + condition.getType() + "</code> is <code>" +
                                    ("greater".equals(condition.getOperator()) ? "greater than" : "") +
                                    ("less".equals(condition.getOperator()) ? "less than" : "") +
                                    ("equal".equals(condition.getOperator()) ? "equal to" : "") +
                                    ("notequal".equals(condition.getOperator()) ? "NOT equal to" : "") +
                                    ("greaterorequal".equals(condition.getOperator()) ? "is greater than or equal to" : "") +
                                    ("lessorequal".equals(condition.getOperator()) ? "is less than or equal to" : "") +
                                    " <code>" + condition.getValue() + "</code>");
                        }
                        if (j < conditions.size() - 1) {
                            println("<code>" + condition.getNext() + "</code>");
                        }
                        println("</li>");

                    }
                    println("</ol>");
                }
                if (!rule.isLast()) {
                    println("<p>Note, other rules will be proessed after this rule.</p>");
                }
                println("</p>");
            }
        }
    }

    private void showRequestInfo(final FormattedCharArrayWriter writer, final HttpServletRequest hsRequest) throws IOException {
        // other info
        println("<h2>Generic Info");
        println("method: " + hsRequest.getMethod());
        println("protocol: " + hsRequest.getProtocol());
        println("port: " + hsRequest.getServerPort());

        // show headers from request
        println("<h2>Your Headers");
        final Enumeration headers = hsRequest.getHeaderNames();
        while (headers.hasMoreElements()) {
            final String headerName = (String) headers.nextElement();
            // ignore cookies as they are handled later
            if ("cookie".equals(headerName)) continue;
            println(headerName + ": " + hsRequest.getHeader(headerName));
        }

        println("<h2>Cookies");
        final Cookie[] cookies = hsRequest.getCookies();
        if (cookies != null) {
            if (cookies.length > 0) {
                for (int i = 0; i < cookies.length; i++) {
                    println("cookie " + i);
                    final Cookie cookie = cookies[i];
                    println("    name     : " + cookie.getName());
                    println("    value    : " + cookie.getValue());
                    println("    path     : " + cookie.getPath());
                    println("    domain   : " + cookie.getDomain());
                    println("    max age  : " + cookie.getMaxAge());
                    println("    is secure: " + cookie.getSecure());
                    println("    version  : " + cookie.getVersion());
                    println("    comment  : " + cookie.getComment());
                }
            } else {
                println("no cookies");
            }
        } else {
            println("no cookies");
        }
    }

    public String bufferToString() {
        return buffer.toString();
    }
}



