/**
 * Copyright (c) 2004, Paul Tuckey
 * All rights reserved.
 * ====================================================================
 * Licensed under the BSD License. Text as follows.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   - Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   - Neither the name tuckey.org nor the names of its contributors
 *     may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
package org.tuckey.web.filters.urlrewrite;

import org.tuckey.utils.FormattedCharArrayWriter;
import org.tuckey.utils.Log;
import org.tuckey.utils.NumberUtils;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.Date;

/**
 * Based on the popular and very useful mod_rewrite for apache, UrlRewriteFilter is a Java Web Filter for any J2EE
 * compliant web application server (such as Resin or Tomcat), which allows you to rewrite URLs before they get to your
 * code. It is a very powerful tool just like Apache's mod_rewrite.
 * <p/>
 * The main things it is used for are:
 * <p/>
 * <ul>
 * <li>URL Tidyness - keep URLs tidy irrespective of the underlying technology (JSPs, servlets, struts etc).</li>
 * <li>Browser Detection - Allows you to rewrite URLs based on request HTTP headers (such as "user-agent").</li>
 * <li>Date based rewriting - Allows you to forward or redirect to other URL's based on the date/time.</li>
 * </ul>
 * UrlRewriteFilter uses an xml file, called urlrewrite.xml (lives in the WEB-INF directory), for configuration. Most
 * parameters can be Perl5 style Regular Expressions. This makes it very powerful indeed.
 * <p/>
 * todo: find mod_rewrite examples and emulate them in docs to help newbies
 * todo: better status page html version (ability to test new conf without having to reload)
 * todo: dont forget to update version
 *
 * @author Paul Tuckey
 * @version $Revision: 1.4 $ $Date: 2004/05/25 07:37:15 $
 */
public final class UrlRewriteFilter implements Filter {

    private static Log log = Log.getLog(UrlRewriteFilter.class);

    public static final String VERSION = "1.2";

    public static final String DEFAULT_WEB_CONF_PATH = "/WEB-INF/urlrewrite.xml";

    /**
     * The conf for this filter.
     */
    private UrlRewriter urlRewriter = null;

    /**
     * A user defined setting that can enable conf reloading.
     */
    private boolean confReloadCheckEnabled = false;

    /**
     * A user defined setting that says how often to check the conf has changed.
     */
    private int confReloadCheckInterval = 0;

    /**
     * The last time that the conf file was loaded.
     */
    private long confLastLoad = 0;

    private long confReloadLastCheck = 0;

    /**
     * Flag to make sure we don't bog the filter down during heavy load.
     */
    private boolean confReloadInProgress = false;

    private boolean statusEnabled = true;
    private String statusPath = "/rewrite-status";


    /**
     *
     */
    private ServletContext context = null;

    /**
     * Init is called automatically by the application server when it creates this filter.
     * 
     * @param filterConfig The config of the filter
     */
    public void init(final FilterConfig filterConfig) {

        // set the conf of the logger to make sure we get the messages in context log
        Log.setConfiguration(filterConfig);

        log.debug("filter init called");

        context = filterConfig.getServletContext();

        // get init paramerers from context web.xml file
        String confReloadCheckIntervalStr = filterConfig.getInitParameter("confReloadCheckInterval");
        String statusPathConf = filterConfig.getInitParameter("statusPath");
        String statusEnabledConf = filterConfig.getInitParameter("statusEnabled");

        // confReloadCheckInterval (default to null)
        if (confReloadCheckIntervalStr != null && !"".equals(confReloadCheckIntervalStr)) {
            // convert to millis
            confReloadCheckInterval = 1000 * NumberUtils.stringToInt(confReloadCheckIntervalStr);
            confReloadCheckEnabled = true;
            if (confReloadCheckInterval == 0) {
                log.info("reload check performed each request");
            } else {
                log.info("reload check set to " + confReloadCheckInterval / 1000);
            }
        } else {
            confReloadCheckEnabled = false;
        }

        // status enabled (default true)
        if (statusEnabledConf != null && !"".equals(statusEnabledConf)) {
            log.debug("statusEnabledConf set to " + statusEnabledConf);
            statusEnabled = "true".equals(statusEnabledConf.toLowerCase());
        }
        if (statusEnabled) {
            // status path (default /rewrite-status)
            if (statusPathConf != null && !"".equals(statusPathConf)) {
                statusPath = statusPathConf.trim();
                log.info("status display enabled, path set to " + statusPath);
            }
        } else {
            log.info("status display disabled");
        }

        loadConf();
    }

    private void loadConf() {
        InputStream inputStream = context.getResourceAsStream(DEFAULT_WEB_CONF_PATH);
        if (inputStream == null) {
            log.error("unable to find urlrewrite conf file at " + DEFAULT_WEB_CONF_PATH);
            // set the writer back to null
            if (urlRewriter != null) {
                log.error("unloading existing conf");
                confLastLoad = 0;
                urlRewriter = null;
            }

        } else {
            confLastLoad = new Date().getTime();
            Conf conf = new Conf(inputStream, "urlrewrite.xml");
            if (log.isDebugEnabled()) {
                if (conf != null && conf.getRules() != null) {
                    log.debug("inited with " + conf.getRules().size() + " rules");
                }
                log.debug("conf is " + (conf.isOk() ? "ok" : "NOT ok"));
            }
            if (conf != null && conf.isOk()) {
                urlRewriter = new UrlRewriter(conf);
                log.info("loaded (conf ok)");

            } else {
                log.error("Conf failed to load");
                urlRewriter = null;
                if (urlRewriter != null) {
                    log.error("unloading existing conf");
                    confLastLoad = 0;
                    urlRewriter = null;
                }
            }
        }
    }

    /**
     * Destroy is called by the application server when it unloads this filter.
     */
    public void destroy() {
        log.info("destroy called");
        context = null;
        urlRewriter = null;
        confLastLoad = 0;
        confReloadCheckEnabled = false;
        confReloadCheckInterval = 0;
        confReloadInProgress = false;
        Log.resetAll();
    }

    /**
     * The main method called for each request that this filter is mapped for.
     *
     * @param request
     * @param response
     * @param chain
     * @throws IOException
     * @throws ServletException
     */
    public void doFilter(final ServletRequest request, final ServletResponse response, final FilterChain chain)
            throws IOException, ServletException {

        // check to see if the conf needs reloading
        long now = new Date().getTime();
        if (confReloadCheckEnabled && !confReloadInProgress &&
                (now - confReloadCheckInterval) > confReloadLastCheck) {
            confReloadInProgress = true;
            confReloadLastCheck = now;

            log.debug("starting conf reload check");
            long confFileCurrentTime = getConfFileLastModified();
            if (confLastLoad < confFileCurrentTime) {
                // reload conf
                log.info("conf file modified since last load, reloading");
                loadConf();
            } else {
                log.debug("conf is not modified");
            }

            confReloadInProgress = false;
        }

        final HttpServletRequest hsRequest = (HttpServletRequest) request;
        final HttpServletResponse hsResponse = (HttpServletResponse) response;

        boolean requestRewritten = false;
        if (urlRewriter != null) {

            // check for status request
            if (statusEnabled) {
                String uri = hsRequest.getRequestURI();
                if (log.isDebugEnabled()) {
                    log.debug("checking for status path on " + uri);
                }
                String contextPath = hsRequest.getContextPath();
                if (uri != null && uri.startsWith(contextPath + statusPath)) {
                    showStatus(hsRequest, response);
                    return;
                }
            }

            // process the request
            RewrittenUrl rewrittenRequest = urlRewriter.processRequest(hsRequest, hsResponse);
            if (rewrittenRequest != null) {
                String toUrl = rewrittenRequest.getTarget();
                if (rewrittenRequest.isForward()) {
                    final RequestDispatcher rq = hsRequest.getRequestDispatcher(toUrl);
                    if (rq == null) {
                        // todo: figure out if this is 404???
                        throw new ServletException("unable to get request dispatcher (for forwarding) for " + toUrl);
                    }
                    rq.forward(hsRequest, response);
                    requestRewritten = true;

                } else if (rewrittenRequest.isInclude()) {
                    final RequestDispatcher rq = hsRequest.getRequestDispatcher(toUrl);
                    if (rq == null) {
                        // todo: figure out if this is 404???
                        throw new ServletException("unable to get request dispatcher (for including) for " + toUrl);
                    }
                    rq.include(hsRequest, hsResponse);
                    chain.doFilter(hsRequest, hsResponse);
                    requestRewritten = true;

                } else if (rewrittenRequest.isRedirect()) {
                    hsResponse.sendRedirect(toUrl);
                    requestRewritten = true;

                } else if (rewrittenRequest.isTemporaryRedirect()) {
                    hsResponse.setStatus(HttpServletResponse.SC_MOVED_TEMPORARILY);
                    hsResponse.setHeader("Location", toUrl);
                    requestRewritten = true;

                } else if (rewrittenRequest.isPermanentRedirect()) {
                    hsResponse.setStatus(HttpServletResponse.SC_MOVED_PERMANENTLY);
                    hsResponse.setHeader("Location", toUrl);
                    requestRewritten = true;

                }
            }
        }

        // if no rewrite has taken place continue as normal
        if (!requestRewritten) {
            chain.doFilter(hsRequest, response);
        }
    }

    /**
     * Gets the last modified date of the conf file.
     *
     * @return time as a long
     */
    private long getConfFileLastModified() {
        File confFile = new File(context.getRealPath(DEFAULT_WEB_CONF_PATH));
        return confFile.lastModified();
    }


    /**
     * Show the status of the conf and the filter to the user.
     *
     * @param request
     * @param response
     * @throws java.io.IOException
     */
    private void showStatus(final HttpServletRequest request, final ServletResponse response)
            throws IOException {

        log.debug("showing status");
        final FormattedCharArrayWriter writer = new FormattedCharArrayWriter();
        Status.displayStatus(writer, request, urlRewriter);

        response.setContentLength(writer.toString().length());

        final PrintWriter out = response.getWriter();
        out.write(writer.toString());
        out.close();

    }

}

