/**
 * Copyright (c) 2004, Paul Tuckey
 * All rights reserved.
 * ====================================================================
 * Licensed under the BSD License. Text as follows.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   - Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   - Neither the name tuckey.org nor the names of its contributors
 *     may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
package org.tuckey.web.filters.urlrewrite;

import org.tuckey.utils.Log;
import org.tuckey.utils.StringUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * The main rewriter.
 *
 * @author Paul Tuckey
 * @version $Revision: 1.4 $ $Date: 2004/05/25 07:37:38 $
 *          <p/>
 *          <p/>
 *          <p/>
 *          > It seems you need to hardcode the context path in your redirect rules.
 *          > Would be nice if a string such as "${context}" were interpolated, or any
 *          > paths not starting with a "/" were considered to be context-relative.
 *          >
 *          > Should be able to choose "moved temporarily" (302) and "moved
 *          > permanently" (301).
 */
public class UrlRewriter {

    private static Log log = Log.getLog(UrlRewriter.class);

    /**
     * The conf for this filter.
     */
    private Conf conf;

    public UrlRewriter(Conf conf) {
        this.conf = conf;
    }


    /**
     * The main method called for each request that this filter is mapped for.
     *
     * @return returns true when request has been handled by url rewriter false when
     */
    public RewrittenUrl processRequest(final HttpServletRequest hsRequest, final HttpServletResponse hsResponse) {

        String originalUrl = StringUtils.trim(hsRequest.getRequestURI());

        if (log.isDebugEnabled()) {
            log.debug("processing request for " + originalUrl);
        }

        if (originalUrl == null) {
            // for some reason the engine is not giving us the url
            // this isn't good
            log.debug("unable to fetch request uri from request.  This shouldn't happen, it may indicate that " +
                    "the web application server has a bug or that the request was not pased correctly.");
            return null;
        }

        // check for context path and remove if it's there
        String contextPath = hsRequest.getContextPath();
        if (contextPath != null && originalUrl.startsWith(contextPath)) {
            log.debug("context stripped");
            originalUrl = originalUrl.substring(contextPath.length());
        }

        // add the query string on uri (note, some web app containers to this)
        if (originalUrl.indexOf("?") == -1) {
            String query = hsRequest.getQueryString();
            if (query != null) {
                query = query.trim();
                if (query.length() > 0) {
                    originalUrl = originalUrl + "?" + query.trim();
                }
            }
        }
        if (log.isDebugEnabled()) {
            log.debug("url " + originalUrl);
        }


        if (!conf.isOk()) {
            // when conf cannot be loaded for some sort of error
            // continue as normal without looking at the non-existent rules
            log.debug("configuration is not ok.  not rewriting request.");
            return null;
        }

        final List rules = conf.getRules();
        if (rules.size() == 0) {
            // no rules defined
            log.debug("there are no rules setup.  not rewriting request.");
            return null;
        }

        // attempt to match the rules
        String finalToUrl = originalUrl;
        RewrittenUrl finalRewrittenRequest = null;
        for (int i = 0; i < rules.size(); i++) {
            final Rule rule = (Rule) rules.get(i);
            final RewrittenUrl rewrittenUrl = rule.execute(finalToUrl, hsRequest, hsResponse);
            if (rewrittenUrl != null) {
                // means this rule has matched
                if (log.isDebugEnabled()) {
                    log.debug("rule " + i + " matched");
                }
                finalRewrittenRequest = rewrittenUrl;
                finalToUrl = rewrittenUrl.getTarget();
                if (rule.isLast()) {
                    log.debug("rule is last");
                    // there can be no more matches on this request
                    break;
                }
            }
        }
        return finalRewrittenRequest;
    }

    public Conf getConf() {
        return conf;
    }


}
